@@ -4,7 +4,7 @@ import json
 import os
 import shutil
 import subprocess
-from typing import Tuple
+from typing import Tuple, Any
 
 import yaml
 
@@ -74,7 +74,7 @@ def runNabu(cfg, repo,glcon="~/indexing/glcon"):
 
 
 
-def getGleaner( cfgfile):
+def getGleaner( cfgfile) -> Tuple[str,str,Any]:
     with open(cfgfile, "r") as stream:
         try:
             cfg =(yaml.safe_load(stream))
@@ -102,7 +102,7 @@ def getSourcesFromGleaner(cfgfile, sourcename=None):
     if sourcename is None:
         return sources
     else:
-        matched = list(filter(lambda source: source.name == sourcename, sources))
+        matched = list(filter(lambda source: source.get('name') == sourcename, sources))
         if len(matched) == 1:
             return matched[0]
         else:
@@ -112,11 +112,11 @@ def getSourcesFromGleaner(cfgfile, sourcename=None):
 def getSitemapSourcesFromGleaner(cfgfile, sourcename=None):
     s3endpoint,  bucket, cfg = getGleaner(cfgfile)
     sources = cfg['sources']
-    sources =list(filter(lambda source: source.sourcetype == "sitemap", sources))
+    sources = list(filter(lambda source: source.get('sourcetype') == "sitemap", sources))
     if sourcename is None:
         return sources
     else:
-        matched = list(filter(lambda source: source.name == sourcename, sources))
+        matched = list(filter(lambda source: source.get('name') == sourcename, sources))
         if len(matched) == 1:
             return matched[0]
         else:
@@ -185,4 +185,4 @@ def _validateJSON(jsonData):
         json.loads(jsonData)
     except ValueError as err:
         return False, err
-    return True, None
\ No newline at end of file
+    return True, None
