@@ -1,35 +1,57 @@
 import json
 import logging
+from typing import Union, Tuple, Any
 
 from jsonschema import validate
 from jsonschema.exceptions import ValidationError
 from string import Template
 
-import pkg_resources
+try:
+    import importlib.resources as pkg_resources
+except ImportError:
+    # Try backported to PY<37 `importlib_resources`.
+    import importlib_resources as pkg_resources
+
+from pydash import ends_with, sort
 from pyld import jsonld
 import ec.sos_json.schemas as schemafiles
 
 jsonld_context = context = {"@vocab": "https://schema.org/"}
 
 
-def _getSchemaFromResources(filename):
-    """ retrieves sparql file from the sparql_files folder when in a package"""
+def _getSchemaFromResources(filename) :
+    """ retrieves json schema file from the  ec.sos_json.schemas directory when in a package"""
     resourcename = f"{filename}.json"
     resource = pkg_resources.read_text(schemafiles, resourcename)
-    schema = json.load(resource)
+    schema = json.loads(resource)
     return schema
 
 
-def isValidJSON(jsonData):
+def isValidJSON(jsonData: str)-> bool:
+    """Is Json Valid"""
     try:
         json.loads(jsonData)
     except ValueError as err:
         return False
     return True
 
+def listSchemaFilesFromResources() -> str:
+    """ retrieves schema file from the schemas folder when in a package"""
+    resource = pkg_resources.contents(schemafiles)
+    files = filter( lambda f: ends_with(f, ".json"), resource)
+    files = sort(list(files))
+    return files
+def validateJson2Schema(json_data: Any, schemaname='GeoCodes-DatasetSchema') -> Tuple[bool,str]:
+    """REF: https://json-schema.org/
 
-def validateJson2Schema(json_data, schemaname='GeoCodes-DatasetSchema.json'):
-    """REF: https://json-schema.org/ """
+    Parameters:
+        json_data: JSON Object (aka json.loads())
+        schemaname: name of schem in ec.sos_json.shcemas: (GeoCodes-DatasetSchema or GeoCodes-ECRR-DatasetSchema.json)
+
+    Returns:
+        bool: is valid
+        message: "Given JSON data is Valid" or "Given JSON data is InValid"
+        """
     # Describe what kind of json you expect.
     execute_api_schema = _getSchemaFromResources(schemaname)
 
@@ -37,14 +59,15 @@ def validateJson2Schema(json_data, schemaname='GeoCodes-DatasetSchema.json'):
         validate(instance=json_data, schema=execute_api_schema)
     except ValidationError as err:
         logging.error(err)
-        err = "Given JSON data is InValid"
+        err = f"Given JSON data is InValid {err}"
         return False, err
 
     message = "Given JSON data is Valid"
     return True, message
 
 
-def validateSosDataset(jsonData):
+def validateSosDataset(jsonData: str) -> bool:
+    """ validates a SOS Dataset"""
     try:
         json_data = json.loads(jsonData)
     except ValueError as err:
@@ -53,7 +76,19 @@ def validateSosDataset(jsonData):
     return valid2_schema
 
 
-def validateEcrrTool(jsonData):
+def validateEcrrTool(jsonData : str) -> bool:
+    """
+       validated
+        Parameters:
+            jsonData: jsonld string returns the string
+            form: 'compact'--compact form,
+                   'frome'--framed usngin schemaType,
+                   'jsonld' - passed form
+            schemaType: type of the schema default="Dataset"
+        Returns:
+            bool.
+
+    """
     try:
         json_data = json.loads(jsonData)
     except ValueError as err:
@@ -63,6 +98,10 @@ def validateEcrrTool(jsonData):
 
 
 def compact_jld_str(jld_str: str) -> str:
+    """
+       depreciated use formatted_jsonld
+
+    """
     doc = json.loads(jld_str)
     compacted = jsonld.compact(doc, jsonld_context)
     r = json.dumps(compacted, indent=2)
@@ -70,6 +109,18 @@ def compact_jld_str(jld_str: str) -> str:
 
 
 def formatted_jsonld(jld_str: str, form="compact", schemaType="Dataset") -> str:
+    """
+       returns a formatted JSONLD string based on the input
+        Parameters:
+            jld_str: jsonld string returns the string
+            form: 'compact'--compact form,
+                   'frome'--framed usngin schemaType,
+                   'jsonld' - passed form
+            schemaType: type of the schema default="Dataset"
+        Returns:
+            string.
+
+    """
     if (form == 'jsonld'):
         return jld_str
 
@@ -90,6 +141,9 @@ def formatted_jsonld(jld_str: str, form="compact", schemaType="Dataset") -> str:
 
         frame_doc = json.loads(thsGraphQuery)
         doc = json.loads(jld_str)
+ #       doc = jsonld.flatten(doc)
+        # issue where @id on Dataset and @id on Dataset.identifer were the same. If so
+        # @type = ['Dataset', 'PropertyValue'] was the conversion.
 
         framed = jsonld.frame(doc, frame_doc)
 
